// Copyright (C) 2015 The Android Open Source Project
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#pragma once

#include "aemu/base/files/PathUtils.h"
#include "aemu/base/system/System.h"
#include "aemu/base/threads/Thread.h"

#include "TestTempDir.h"

namespace android {
namespace base {

class TestSystem {
public:
    TestTempDir* getTempRoot() const {
        if (!mTempDir) {
            mTempDir = new TestTempDir("TestSystem");
            mTempRootPrefix = PathUtils::addTrailingDirSeparator(mTempDir->pathString());
        }
        return mTempDir;
    }

    ~TestSystem() {
        if (mTempDir) delete mTempDir;
    }
private:

    mutable TestTempDir* mTempDir = nullptr;
    mutable std::string mTempRootPrefix;
};

}  // namespace base
}  // namespace android
