// Copyright 2017 The Android Open Source Project
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#pragma once

// Include this for the definition of enums missing on different platforms.
#include "android/utils/fd.h"

namespace android {
namespace base {

// Make a file descriptor "close-on-exec" to make sure it won't get inherided
// by a fork-execed child.
void fdSetCloexec(int fd);

}  // namespace base
}  // namespace android
