// Copyright 2016 The Android Open Source Project
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#pragma once

#include "aemu/base/async/Looper.h"

#include <memory>

namespace android {
namespace base {

// ScopedSocketWatch is a convenience class that implements a scoped
// Looper::FdWatch pointer that will automatically call socketClose()
// on destruction.

// Custom deleter from std::unique_ptr
struct SocketWatchDeleter {
    void operator()(android::base::Looper::FdWatch* watch) const;
};

using ScopedSocketWatch = std::unique_ptr<Looper::FdWatch, SocketWatchDeleter>;

}  // namespace base
}  // namespace android

