// Copyright 2019 The Android Open Source Project
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#pragma once

namespace android {
namespace base {

class JsonWriter;

} // namespace base
} // namespace android

namespace android {
namespace perflogger {

class Analyzer {
public:
    enum MetricAggregate {
        Mean,
        Median,
        Min,
        Max,
    };

    enum DirectionBias {
        NoBias,
        IgnoreIncrease,
        IgnoreDecrease,
    };

    virtual void outputJson(base::JsonWriter*) { }
};

} // namespace perflogger
} // namespace android